#include "asm86.h"
#include "Ram86.inc"

_mov10B			equ		$427b
textflags			equ		5
textInverse			equ		3

.org _asm_exec_ram 

 ld a,1					;creating one new token
 ld (_numtok_asap3),a

;The calculator has what seems to be called an "asap" that makes all this possible.  When
;creating a token, you need you only of the options of 3 asap's (I have used Ram86.inc to make
;this easier to see), and I am using the third asap because stbeta1 and finbeta1 may use the 
;other two (they use which ever one's not occupied).  However, each asap is not limited to one 
;token, and so the byte at (_numtok_asapX)* will identify how many.  
;* X = 1,2 or 3

 ld de,$f8ce				;puts tokspell here
 ld (_tokspell_asap3),de		;(_tokspell_asap3) = pointer to tokspell
 ld hl,tokspell
 call _mov10B				;copy tokspell to $f8ce

;Next, the OS needs to be able to recognize the token, so we have to put a pointer at 
;(_tokspell_asapX).  I pointed to $f8ce.  At $f8ce I need another pointer to the actual name as 
;it is spelled.  If we had more than one token, we would list all the pointers (two bytes each)
;at $f8ce--because the calc already knows how many tokens we're creating, it will know when to 
;stop.  I put "Invert",0 right after these pointers so that I could move everything more easily
;and efficiently.  Then, as follows, the spelling starts with a zero-terminated string.  It 
;continues with a byte indicating which token number it is (I have $01 because it is my first
;and only token) and the last byte identifies some kind of type (this type byte is NOT the same
;as the byte used in menus); you should put $80 as the type.

 ld de,$f8d8				;puts the table here
 ld (_eostbl_asap3),de			;(_eostbl_asap3) = pointer to the table
 ld hl,table
 call _mov10B

;The "eos" table contains the two-byte address of the assembly code (remember that the bytes are
;reversed),the type (straight assembly = $01), and a null.  This table should list all tokens 
;in the order that was specified in the spelling (four bytes for each token).  


 ld hl,asapname
 ld de,_asap_nl3
 call _mov10B

;I don't know what this does, but you'll get an *UNDEFINED* error if you don't do it.  Its 
;probably for the reinstall asap, which will reinstall the assembly code onto the RAM page.
;This is in case another assembly program uses the same memory; it most likely reinstalls after
;an *Asm(* command is executed.  I don't know how to use this asap, so I didn't.


custom_menu:
 ld de,$d20b
 ld ($d1cd),de				;puts pointer ($d2ob) in the place that indicates
						;5th menu -- actual entry is stored at $d20b						
 ld hl,custom				;puts $00,"Invrt" into the custom menu
 call _mov10B

;When the calculator reads a menu (in this case, the custom menu at $d1c3 when [CUSTOM] is
:pushed), it sees a byte indicating whether to bring up a sub-menu ($08) or a byte to bring
;up a new menu ($09).  Next, it reads a byte telling how many menu positions are in that menu.
;Note that if (in the custom menu only) these two bytes are anything besides $09,$0f, they will
;be changed back the next time that you enter the catalog.
;On the third byte the pointers start.  They are two bytes each and follow one another in order,
;according to how many how many menu positions are specified in the second byte.  Each pointer
;points to the actual entry.  The catalog spaces the entries ten bytes apart but you can make
;the pointer point anywhere on the static RAM page if that isn't long enough.  The entry
;contains one byte that identifies the type followed by a zero terminated string.  Because the
;custom menu entries are ten bytes each, the call "_mov10B" works nicely and I think that this 
;is what the catalog uses to place entries.  Here are some other entries, not included in the 
;"86custom" document:

;$06, followed by anything, does nothing, if for some reason you put a pointer there

;$05, followed by a two byte pointer, and a zero terminated string will display the string
;	in the entry slot and tell the calculator calculator that the bytes at that pointer will
;	be some kind of menu (a menu type byte, a number of entries byte, followed by pointers)

;$01 will execute assembly, it is followed by two bytes (I don't know what these two bytes do,
;	but I don't think it matters; you can just put $00,$00), which is followed by $c3 (= jp
;	$XX XX), followed then by the pointer to the code (if I wanted it to run my invert code,
;	I would put $00,$f2), and finally the zero terminated string.
;	(this will probably be more than 10 bytes, so if you want to put it in the Custom menu
;	you will have to change the pointers.)

;$41 does the same as $01

;$03 does something; I'll try to figure it out  :?)



;loads code into $f200
load_code:
 ld hl,code
 ld de,$f200			
 ld bc,code_end-code
 ldir
 ret


;invert function
code:
 bit textInverse,(iy+textflags)
 jr z,set
reset:
 res textInverse,(iy+textflags)
 ret
set:
 set textInverse,(iy+textflags)
 ret
code_end:

tokspell:
.db $d0,$f8					;pointer to "Invert",0
actual_spelling:
.db "Invrt",0
.db $01,$80
tokspell_end:

custom:
.db $00					;pastes the invert to homescreen exactly as it is
.db "Invrt",0

table:
.db $00,$f2,$01,$00			;the "eos" table starts with a two byte pointer to the code
						;and then $01,$00

asapname:
.db $07					;number of letters in the name
.db "asapinv",0				;the name

.end
.end


;Another "asap" is for the MATH menu.  You can look in Ram86.inc for the pointers; it works
;in the same way as the menus I described above, except that the first pointers are always 
;present.


;Dux Gregis
;assets@eden.rutgers.edu
